/* provide encrypted configuration file on stdin, */
/* get decrypted configuration file on stdout. */
/* program code adopted from: */
/* http://wiki.openwrt.org/toh/arcadyan/arv7506 "Decrypting configuration backup" */
/* program code adapted for Arcor/Vodafone Easyboxes by scolopender */

import java.security.MessageDigest;
import javax.crypto.Cipher;
import javax.crypto.spec.*;
import java.io.*;
import java.util.*;

class XtractEbCfg
{
  static final byte[] key0 =
  {
    (byte)0x1d,(byte)0xe5,(byte)0xfa,(byte)0xb8,(byte)0x81,(byte)0x91,(byte)0x00,(byte)0x84,
    (byte)0x11,(byte)0xaf,(byte)0x53,(byte)0xdd,(byte)0xe4,(byte)0x89,(byte)0xea,(byte)0xfd
  };
  static final byte[] key1 =
  {
    (byte)0xa9,(byte)0xdc,(byte)0x0e,(byte)0xf0,(byte)0x28,(byte)0x67,(byte)0x91,(byte)0xac,
    (byte)0xc9,(byte)0x13,(byte)0x86,(byte)0x5d,(byte)0x2d,(byte)0xf8,(byte)0x20,(byte)0x99
  };
  static final byte[] key2 =
  {
    (byte)0x1a,(byte)0xa8,(byte)0x80,(byte)0xb5,(byte)0x44,(byte)0x11,(byte)0xf1,(byte)0x19,
    (byte)0x0c,(byte)0xca,(byte)0x3d,(byte)0xe8,(byte)0x77,(byte)0x41,(byte)0x37,(byte)0x89
  };
  static final byte[] key3 =
  {
    (byte)0x45,(byte)0x68,(byte)0x70,(byte)0x98,(byte)0x7a,(byte)0xcd,(byte)0xeb,(byte)0xbb,
    (byte)0xbd,(byte)0xed,(byte)0xa8,(byte)0x03,(byte)0x98,(byte)0x73,(byte)0x40,(byte)0x24
  };
  static final byte[][] keys =
  {
    key0, key1, key2, key3
  };

  static void doit(InputStream source, OutputStream sink) throws Exception
  {
    Cipher c = Cipher.getInstance("AES/CTS/NoPadding");
    byte[] iv = new byte[16];
    source.read(iv);
    c.init(Cipher.DECRYPT_MODE, new SecretKeySpec(key0, "AES"), new IvParameterSpec(iv, 0, 16));
    byte[] header = new byte[0x28];
    source.read(header);
    DataInput hdrstream = new DataInputStream(new ByteArrayInputStream(c.doFinal(header)));
    int headermagic = hdrstream.readInt();
    if ( headermagic != 0x59585756 )
    {
      throw new Exception("Bad header magic: " + headermagic);
    }
    int size = hdrstream.readInt();
    int keynum = hdrstream.readByte();
    if ( (keynum > 3) || (keynum < 0) )
    {
      throw new Exception("Bad key number (0-3 expected): " + keynum);
    }
    byte[] expectedmd5 = new byte[16];
    hdrstream.readFully(expectedmd5);
    byte[] body = new byte[size];
    source.read(body);
    byte[] actualmd5 = MessageDigest.getInstance("MD5").digest(body);
    if ( !MessageDigest.isEqual(actualmd5, expectedmd5) )
    {
      throw new Exception("Body MD5 does not match: " + actualmd5 + expectedmd5);
    }
    c.init(Cipher.DECRYPT_MODE, new SecretKeySpec(keys[keynum], "AES"), new IvParameterSpec(body, 0, 16));
    byte[] payload = c.doFinal(body, 16, size - 16);
    DataInputStream bodystream = new DataInputStream(new ByteArrayInputStream(payload));
    int bodymagic = bodystream.readInt();
    if ( bodymagic != 0x33343536 )
    {
      throw new Exception("Bad body magic: " + bodymagic);
    }
    sink.write(payload);
    sink.flush();
  }

  public static void main(String[] args) throws Exception
  {
    doit(System.in, System.out);
  }
}

/* EoF */
